@@ -380,6 +380,12 @@ Decoder <- R6Class("Decoder",
               "midOffsetAtWhole",
               "midOffsetAtHalf")] <- imsg$pop(13L)
 
+      if(private$serverVersion >= MIN_SERVER_VER_CUSTOMER_ACCOUNT)
+        order$customerAccount <- imsg$pop()
+
+      if(private$serverVersion >= MIN_SERVER_VER_PROFESSIONAL_CUSTOMER)
+        order$professionalCustomer <- imsg$pop()
+
       private$validate("openOrder", orderId=    order$orderId,
                                     contract=   contract,
                                     order=      order,
@@ -430,15 +436,21 @@ Decoder <- R6Class("Decoder",
 
       cd <- ContractDetails
 
-      cd$contract[c(2L:6L, 8L, 10L, 11L)] <- imsg$pop(8L)
-      cd$marketName                       <- imsg$pop()
-      cd$contract$tradingClass            <- imsg$pop()
-      cd$contract$conId                   <- imsg$pop()
-      cd$minTick                          <- imsg$pop()
-      cd$contract$multiplier              <- imsg$pop()
-      cd[4L:8L]                           <- imsg$pop(5L)
-      cd$contract$primaryExchange         <- imsg$pop()
-      cd[9L:17L]                          <- imsg$pop(9L)
+      cd$contract[2L:4L] <- imsg$pop(3L)
+
+      if(private$serverVersion >= MIN_SERVER_VER_LAST_TRADE_DATE)
+        cd$contract$lastTradeDate <- imsg$pop()
+
+      cd$contract[c(5L, 6L, 8L, 10L, 11L)] <- imsg$pop(5L)
+
+      cd$marketName               <- imsg$pop()
+      cd$contract$tradingClass    <- imsg$pop()
+      cd$contract$conId           <- imsg$pop()
+      cd$minTick                  <- imsg$pop()
+      cd$contract$multiplier      <- imsg$pop()
+      cd[4L:8L]                   <- imsg$pop(5L)
+      cd$contract$primaryExchange <- imsg$pop()
+      cd[9L:17L]                  <- imsg$pop(9L)
 
       n <- Validator$i(imsg$pop())
 
@@ -1256,6 +1268,12 @@ Decoder <- R6Class("Decoder",
               "midOffsetAtWhole",
               "midOffsetAtHalf")] <- imsg$pop(5L)
 
+      if(private$serverVersion >= MIN_SERVER_VER_CUSTOMER_ACCOUNT)
+        order$customerAccount <- imsg$pop()
+
+      if(private$serverVersion >= MIN_SERVER_VER_PROFESSIONAL_CUSTOMER)
+        order$professionalCustomer <- imsg$pop()
+
       private$validate("completedOrder", contract=   contract,
                                          order=      order,
                                          orderState= orderState)
